function [A, f, phi, C, res, iter] = harmonic_fit(x, fs, f0, tol)
% HARMONIC_FIT Fit a harmonic function using the IEEE 1057 standard method
%
% [A, F, PHI, C, RES, ITER] = HARMONIC_FIT(X, FS, F0, TOL)
%   Fits the harmonic function in the form
%   x =~ A cos(2*pi*f*t + phi) + C
%   Onto the samples given in the vector X. FS is the sampling frequency
%   and F0 is the initial frequency guess. The iterations of the algorithm 
%   are repeated until the relative difference between the new and the
%   previous parameter vector becomes smaller than the tolerance TOL 
%   (default: 1e-6).
%
%   The fitted parameters are:
%       A    - Amplitude
%       F    - Frequency
%       PHI  - Phase
%       C    - Constant offset
%   Other outputs:
%       RES  - Residual of parameter vector
%       ITER - Number of iterations needed

% Check tolerance
if nargin < 4 || isempty(tol)
    tol = 1e-6;
elseif ~isscalar(tol) || ~isreal(tol) || tol <= 0 || tol >= 1
    error('harmonic_fit:invalid_tol', ...
        'Tolerance parameter TOL must be a positive scalar in the range (0, 1)');
end

% Check x
if ~isvector(x) || ~isreal(x)
    error('harmonic_fit:invalid_x', ...
        'Sample vector X must be a vector with real-valued elements');
else
    x = x(:);
end

% Check fs
if ~isscalar(fs) || ~isreal(fs) || fs <= 0
    error('harmonic_fit:invalid_fs', ...
        'Sampling frequency FS must be a positive scalar value.');
end

% Check f0
if ~isscalar(f0) || ~isreal(f0) || f0 <= 0
    error('harmonic_fit:invalid_f0', ...
        'Initial frequency guess F0 must be a positive scalar value.');
end

n = size(x, 1);
t = (0 : n-1).' / fs;

D0 = [cos(2*pi*f0*t), sin(2*pi*f0*t), ones(size(t))];
p = D0 \ x;
p_prev = [p; 0];

res = inf;
f = f0;
iter = 0;
max_iter = 30;

while res > tol && iter < max_iter
    D = [cos(2*pi*f*t), sin(2*pi*f*t), ones(size(t)), ...
         2*pi*t.*(-p(1)*sin(2*pi*f*t) + p(2)*cos(2*pi*f*t))];
    p = D \ x;
    f = f + p(4);
    
    res = norm(p-p_prev) / norm(p_prev);
    p_prev = p;
    iter = iter + 1;
end

% Warning if tolerance not reached
if res > tol
    warning('harmonic_fit:tolerance_not_reached', ...
        'The specified tolerance %g could not be reached in %d iterations', tol, iter);
end

% Convert to A*cos(2*pi*f*t + phi) format
A = sqrt(p(1)^2 + p(2)^2);
phi = atan2(-p(2), p(1));
C = p(3);

end % of harmonic_fit
