function [Y_mean, f] = spec_aver(x, fs, N_win, N_over, winfun)
% SPEC_AVER Compute one-sided spectral average
%
%   [Y, F] = SPEC_AVER(X, FS, NW) 
%   [Y, F] = SPEC_AVER(X, FS, NW, NO)
%   [Y, F] = SPEC_AVER(X, FS, NW, NO, FUN)
%
%   Computes the average one-sided spectrum of the input signal X. The
%   subsequent samples in X must be in the rows of X. In case of a
%   one-channel sample, X must be a column vector. If X has K columns, then
%   the output Y will also have K columns, each column containing the
%   average spectrum of one channel. FS must be a positive scalar giving 
%   the sampling frequency of the input signal X. NW is the number of 
%   samples used in each analysis window.
%   The optional parameter NO is the number of overlapping samples. NO
%   must be smaller than NW. As a default value NO = NW / 2 is used.
%   The optional parameter FUN is the window function to be used for the
%   analysis. As a default value FUN = @hann is used.
%
%   The result matrix Y will have NW / 2 + 1 rows and the same number of
%   columns as X. The output vector F is a column vector with the same
%   number of rows as Y.
%
%   Example:
%       T = 2;                          % Length of sample [s]
%       fs = 44100;                     % Sampling frequency [Hz]
%       N = ceil(T * fs);               % Number of samples
%       t = (0 : N-1).'/fs;             % Time vector
%       f0 = 1300;                      % Signal frequency [Hz]
%       A = 0.6;                        % Signal amplitude
%       x = A*sin(2*pi*f0*t);           % The signal
%       Nw = 4096;                      % Window size
%       [Y, f] = spec_aver(x, fs, Nw);  % Spectral average
%       plot(f, 20*log10(Y));           % Plot the result

if nargin < 5
    winfun = @hann;
end

if nargin < 4
    N_over = floor(N_win/2);
end

if floor(N_win/2) ~= N_win/2
    error('spec_aver:invalid_window_size', ...
        'Window size parameter NW must be even');
end

N_samp = size(x, 1);
N_channel = size(x, 2);
N_aver = floor((N_samp - N_win) / (N_win-N_over)) + 1;

win = winfun(N_win);
Y_mean = zeros(N_win, N_channel);
for i = 1 : N_aver
    idx = (i-1) * (N_win - N_over) + (1 : N_win);
    Y_mean = Y_mean + abs(fft(bsxfun(@times, x(idx, :), win), [], 1)).^2;
end

enbw = sum(win);
Y_mean = sqrt(Y_mean / N_aver) / enbw;
Y_mean(2 : N_win/2-1) = 2*Y_mean(2 : N_win/2-1);
Y_mean = Y_mean(1 : floor(N_win/2)+1);

f = (0 : floor(N_win/2)).'/N_win*fs;

end % of function
